/* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#if defined(__ANDROID__)

// Allow use of stuff in <time.h>
#define FORBIDDEN_SYMBOL_EXCEPTION_time_h

// Disable printf override in common/forbidden.h to avoid
// clashes with log.h from the Android SDK.
// That header file uses
//   __attribute__ ((format(printf, 3, 4)))
// which gets messed up by our override mechanism; this could
// be avoided by either changing the Android SDK to use the equally
// legal and valid
//   __attribute__ ((format(printf, 3, 4)))
// or by refining our printf override to use a varadic macro
// (which then wouldn't be portable, though).
// Anyway, for now we just disable the printf override globally
// for the Android port
#define FORBIDDEN_SYMBOL_EXCEPTION_printf

#include <sys/time.h>
#include <sys/resource.h>
#include <sys/system_properties.h>
#include <time.h>
#include <unistd.h>
#include "common/stream.h"
#include "common/util.h"
#include "common/textconsole.h"
#include "common/rect.h"
#include "common/queue.h"
#include "common/mutex.h"
#include "common/events.h"
#include "common/config-manager.h"
#include "backends/fs/posix/posix-fs.h"

#include "backends/keymapper/keymapper.h"
#include "backends/saves/default/default-saves.h"
#include "backends/timer/default/default-timer.h"

#include "backends/platform/android/jni.h"
#include "backends/platform/android/android.h"
#include "backends/platform/android/android_common.h"
#include "common/str.h"
#include "common/file.h"


const char *android_log_tag = "ScummVM";
const char* filenameSaveFile1="/sdcard/Android/data/com.dotemu.gobliiins/files/Gob/Gob1/save.dat";
 char* filenameSaveFile2="/sdcard/Android/data/com.dotemu.gobliiins/files/Gob/Gob2/save.dat";
 char* filenameSaveFile3="/sdcard/Android/data/com.dotemu.gobliiins/files/Gob/Gob3/save.dat";
char* filenameSettings="/sdcard/Android/data/com.dotemu.gobliiins/files/Gob/settings.dat";
const char* filenameBitmapFont_black="/sdcard/Android/data/com.dotemu.gobliiins/files/Gob/Gob2/8bitoperator_black.fnt";
const char* filenameBmpFont_black="/sdcard/Android/data/com.dotemu.gobliiins/files/Gob/Gob2/8bitoperator_black.png";
const char* filenameBitmapFont_white="/sdcard/Android/data/com.dotemu.gobliiins/files/Gob/Gob1/8bitoperator_white.fnt";
const char* filenameBmpFont_white="/sdcard/Android/data/com.dotemu.gobliiins/files/Gob/Gob1/8bitoperator_white.png";




bool filter_linear=false;
extern SaveLoadView saveLoadView;
extern unsigned int numSlot;
extern Common::Rect game_screen;
bool bSimu;

dotemuBmpFont _bmpFont_black;
dotemuBmpFont _bmpFont_white;
bool bPause;
bool bSkip;
bool bShowSkipButton;
bool bInGame;
bool bReload;
bool bResume;
extern int currentGame;
bool bFakeMousePause;
Common::Point prePos;
extern bool showHotspot;
extern bool sendEvent;
extern bool nextGob;
extern int kTempoSwitch;
extern bool fakePos;
extern const char* niveau;
extern bool bGoHotspot;
extern int menu_sound;
extern int sfx_sound;


bool fingusBusy;
bool winkleBusy;
int fingusLeft;
int fingusTop;
int winkleLeft;
int winkleTop;
int gob3State;
bool changeGobliin;
int numberGobliin;
int princeLeft;
int princeTop;
int princeLastLeft;
bool WarningMessage;

bool bGoSim;
bool bLoading;
extern int idLvlCourant;
int cpt;
bool bLvLOneLoaded;
bool bRestart;
int bCptSimul;
int cptEsc;
int etat;
int bLoop2;
int bLoop;
bool bSimulCode;
bool bSimulation;
int idLvlToLoad;
bool bGameOverOn;
bool bSimulGO;
bool bSimulGOver;
bool bExit;
bool bBagOn;
int bGo;
bool bLoaded;
Common::Point posSimulation;


extern bool bWrite;

// This replaces the bionic libc assert functions with something that
// actually prints the assertion failure before aborting.
extern "C" {
	void __assert(const char *file, int line, const char *expr) {
		__android_log_assert(expr, android_log_tag,
								"Assertion failure: '%s' in %s:%d",
								 expr, file, line);
	}

	void __assert2(const char *file, int line, const char *func,
					const char *expr) {
		__android_log_assert(expr, android_log_tag,
								"Assertion failure: '%s' in %s:%d (%s)",
								 expr, file, line, func);
	}
}

#ifdef ANDROID_DEBUG_GL
static const char *getGlErrStr(GLenum error) {
	switch (error) {
	case GL_INVALID_ENUM:
		return "GL_INVALID_ENUM";
	case GL_INVALID_VALUE:
		return "GL_INVALID_VALUE";
	case GL_INVALID_OPERATION:
		return "GL_INVALID_OPERATION";
	case GL_STACK_OVERFLOW:
		return "GL_STACK_OVERFLOW";
	case GL_STACK_UNDERFLOW:
		return "GL_STACK_UNDERFLOW";
	case GL_OUT_OF_MEMORY:
		return "GL_OUT_OF_MEMORY";
	}

	static char buf[40];
	snprintf(buf, sizeof(buf), "(Unknown GL error code 0x%x)", error);

	return buf;
}

void checkGlError(const char *expr, const char *file, int line) {
	GLenum error = glGetError();

	if (error != GL_NO_ERROR)
		LOGE("GL ERROR: %s on %s (%s:%d)", getGlErrStr(error), expr, file, line);
}
#endif

OSystem_Android::OSystem_Android(int audio_sample_rate, int audio_buffer_size) :
	_audio_sample_rate(audio_sample_rate),
	_audio_buffer_size(audio_buffer_size),
	_screen_changeid(0),
	_egl_surface_width(0),
	_egl_surface_height(0),
	_htc_fail(true),
	_force_redraw(false),
	_game_texture(0),
	_overlay_texture(0),
	_mouse_texture(0),
	_mouse_texture_palette(0),
	_mouse_texture_rgb(0),
	_mouse_hotspot(),
	_mouse_keycolor(0),
	_use_mouse_palette(false),
	_graphicsMode(0),
	_waitShowWarning(0),
	_fullscreen(true),
	_ar_correction(true),
	_show_mouse(false),
	_show_overlay(false),
	_enable_zoning(false),
	_mixer(0),
	_shake_offset(0),
	_queuedEventTime(0),
	_event_queue_lock(createMutex()),
	_touch_pt_down(),
	_touch_pt_scroll(),
	_touch_pt_dt(),
	_eventScaleX(100),
	_eventScaleY(100),
	// TODO put these values in some option dlg?
	_touchpad_mode(false),
	_touchpad_scale(66),
	_dpad_scale(4),
	_fingersDown(0),
	_hotspot_texture(0),
	_border_texture(0),
	_bg_texture(0),
	_loading_texture(0),
	_menu_texture(0),
	
	_texte_texture(0),	
	
	_gameover_texture(0),
	
	_popup_texture(0),
	
	
	
	_waitExit(0),
	_delayHotspot(0),
	_oldPos(Common::Point(0,0)),
	_oldPosReel(Common::Point(0,0)),
	
	_trackball_scale(2) {
	
	showHotspot=false;
	bGo=-1;
	sendEvent=false;
	nextGob=false;
	fakePos=false;
	bFakeMousePause=false;
	prePos=Common::Point(0,0);
	bSimu=false;
	kTempoSwitch=12;
	bPause=false;
	bInGame=false;
	bGameOverOn=false;
	bReload=false;
	bExit=false;
	bSimulGO=false;
	bSimulGOver=false;
	bGoHotspot=false;
	bLoaded=false;
	bResume=false;
	bGoSim=false;
	bBagOn=false;
	WarningMessage=false;
	
	bRestart=false;
	bLvLOneLoaded=true;
	bSimulCode=false;
	bSimulation = false;
	bCptSimul=0;
	bLoop=-1;
	bLoop2=-1;
	etat=-1;
	cptEsc=0;
	cpt=0;
	niveau="";
	idLvlToLoad=0;
	bWrite=false;
	bSkip=false;
	bLoading=false;
	bShowSkipButton=false;
	posSimulation=Common::Point(121,113);
	
	fingusBusy=false;
	winkleBusy=false;
	fingusLeft=0;
	fingusTop=0;
	winkleLeft=0;
	winkleTop=0;
	gob3State=0;
	changeGobliin=false;
	numberGobliin=1;
	princeLeft=0;
	princeTop=0;
	princeLastLeft=0;

	
	_fsFactory = new POSIXFilesystemFactory();

	

	Common::String mf = getSystemProperty("ro.product.manufacturer");

	LOGI("Running on: [%s] [%s] [%s] [%s] [%s] SDK:%s ABI:%s",
			mf.c_str(),
			getSystemProperty("ro.product.model").c_str(),
			getSystemProperty("ro.product.brand").c_str(),
			getSystemProperty("ro.build.fingerprint").c_str(),
			getSystemProperty("ro.build.display.id").c_str(),
			getSystemProperty("ro.build.version.sdk").c_str(),
			getSystemProperty("ro.product.cpu.abi").c_str());

	mf.toLowercase();
	/*_htc_fail = mf.contains("htc");

	if (_htc_fail)
		LOGI("Enabling HTC workaround");*/
}

OSystem_Android::~OSystem_Android() {
	ENTER();

	delete _mixer;
	_mixer = 0;
	delete _hotspot_texture;
	_hotspot_texture=0;
	
	delete _gameover_texture;
	_gameover_texture=0;
	
	delete _popup_texture;
	_popup_texture=0;
	
	
	delete _loading_texture;
	_loading_texture=0;
	

	
	delete _texte_texture;
	_texte_texture=0;
	

	delete _fsFactory;
	_fsFactory = 0;
	delete _timerManager;
	_timerManager = 0;

	deleteMutex(_event_queue_lock);
}

void *OSystem_Android::timerThreadFunc(void *arg) {
	OSystem_Android *system = (OSystem_Android *)arg;
	DefaultTimerManager *timer = (DefaultTimerManager *)(system->_timerManager);

	// renice this thread to boost the audio thread
	if (setpriority(PRIO_PROCESS, 0, 19) < 0)
		LOGW("couldn't renice the timer thread");

	JNI::attachThread();

	struct timespec tv;
	tv.tv_sec = 0;
	tv.tv_nsec = 10 * 1000 * 1000; // 10ms

	while (!system->_timer_thread_exit) {
		if (JNI::pause) {
			LOGD("timer thread going to sleep");
			sem_wait(&JNI::pause_sem);
			LOGD("timer thread woke up");
		}

		timer->handler();
		nanosleep(&tv, 0);
	}

	JNI::detachThread();

	return 0;
}

void *OSystem_Android::audioThreadFunc(void *arg) {
	
	JNI::attachThread();

	OSystem_Android *system = (OSystem_Android *)arg;
	Audio::MixerImpl *mixer = system->_mixer;

	uint buf_size = system->_audio_buffer_size;

	JNIEnv *env = JNI::getEnv();

	jbyteArray bufa = env->NewByteArray(buf_size);

	bool paused = true;

	byte *buf;
	int offset, left, written;
	int samples, i;

	struct timespec tv_delay;
	tv_delay.tv_sec = 0;
	tv_delay.tv_nsec = 20 * 1000 * 1000;

	uint msecs_full = buf_size * 1000 / (mixer->getOutputRate() * 2 * 2);

	struct timespec tv_full;
	tv_full.tv_sec = 0;
	tv_full.tv_nsec = msecs_full * 1000 * 1000;

	bool silence;
	uint silence_count = 33;


//	LOGI("Enter audioThreadFunc");
	while (!system->_audio_thread_exit) {
//		debugdot("While !audio_thread_exit");
		if (JNI::pause ) {
			JNI::setAudioStop();

			paused = true;
			silence_count = 33;

			LOGD("audio thread going to sleep");
			sem_wait(&JNI::pause_sem);
			LOGD("audio thread woke up");
		}

		buf = (byte *)env->GetPrimitiveArrayCritical(bufa, 0);
		assert(buf);

		samples = mixer->mixCallback(buf, buf_size);

		silence = samples < 1;

		// looks stupid, and it is, but currently there's no way to detect
		// silence-only buffers from the mixer
		if (!silence) {
			silence = true;

			for (i = 0; i < samples; i += 2)
				// SID streams constant crap
				if (READ_UINT16(buf + i) > 32) {
					silence = false;
					break;
				}
		}

		env->ReleasePrimitiveArrayCritical(bufa, buf, 0);

		if (silence || (bSimulation&& idLvlCourant==1) || bGoHotspot) {
			if (!paused)
				silence_count++;

			// only pause after a while to prevent toggle mania
			if (silence_count > 32) {
				if (!paused) {
					LOGD("AudioTrack pause");

					JNI::setAudioPause();
					paused = true;
				}

				nanosleep(&tv_full, 0);

				continue;
			}
		}
	
		if (paused) {
			LOGD("AudioTrack play");

			JNI::setAudioPlay();
			paused = false;

			silence_count = 0;
		}

		offset = 0;
		left = buf_size;
		written = 0;

		while (left > 0) {
		//	debugdot("while left = %d",left);
			written = JNI::writeAudio(env, bufa, offset, left);

			if (written < 0) {
				LOGE("AudioTrack error: %d", written);
				break;
			}

			// buffer full
			if (written < left){
				nanosleep(&tv_delay, 0);
				debugdot("audio write");
			}

			offset += written;
			left -= written;
		}

		if (written < 0)
			break;

		// prepare the next buffer, and run into the blocking AudioTrack.write
	}

	JNI::setAudioStop();

	env->DeleteLocalRef(bufa);

	JNI::detachThread();

	return 0;
}

void OSystem_Android::initBackend() {
	ENTER();

	_main_thread = pthread_self();

	ConfMan.registerDefault("fullscreen", true);
	ConfMan.registerDefault("aspect_ratio", true);

	ConfMan.setInt("autosave_period", 0);
	ConfMan.setBool("FM_high_quality", false);
	ConfMan.setBool("FM_medium_quality", true);

	// TODO hackity hack
	if (ConfMan.hasKey("multi_midi"))
		_touchpad_mode = !ConfMan.getBool("multi_midi");

	// must happen before creating TimerManager to avoid race in
	// creating EventManager
	setupKeymapper();

	// BUG: "transient" ConfMan settings get nuked by the options
	// screen. Passing the savepath in this way makes it stick
	// (via ConfMan.registerDefault)
	_savefileManager = new DefaultSaveFileManager(ConfMan.get("savepath"));
	_timerManager = new DefaultTimerManager();

	gettimeofday(&_startTime, 0);

	_mixer = new Audio::MixerImpl(this, _audio_sample_rate);
	_mixer->setReady(true);

	_timer_thread_exit = false;
	pthread_create(&_timer_thread, 0, timerThreadFunc, this);

	_audio_thread_exit = false;
	pthread_create(&_audio_thread, 0, audioThreadFunc, this);

	initSurface();
	initViewport();

	_game_texture = new GLESFakePalette565Texture();
	_overlay_texture = new GLES4444Texture();
	_mouse_texture_palette = new GLESFakePalette5551Texture();
	_mouse_texture = _mouse_texture_palette;

	initOverlay();

	// renice this thread to boost the audio thread
	if (setpriority(PRIO_PROCESS, 0, 19) < 0)
		warning("couldn't renice the main thread");

	JNI::setReadyForEvents(true);

	EventsBaseBackend::initBackend();
	LOGI("exit initbackend");
}

void OSystem_Android::addPluginDirectories(Common::FSList &dirs) const {
	ENTER();

	JNI::getPluginDirectories(dirs);
}

bool OSystem_Android::hasFeature(Feature f) {
	return (f == kFeatureFullscreenMode ||
			f == kFeatureAspectRatioCorrection ||
			f == kFeatureCursorPalette ||
			f == kFeatureVirtualKeyboard ||
			f == kFeatureOverlaySupportsAlpha);
}

void OSystem_Android::setFeatureState(Feature f, bool enable) {
	ENTER("%d, %d", f, enable);
	
	switch (f) {
	case kFeatureFullscreenMode:
		_fullscreen = enable;
		updateScreenRect();
		break;
	case kFeatureAspectRatioCorrection:
		_ar_correction = enable;
		updateScreenRect();
		break;
	case kFeatureVirtualKeyboard:
		//debugdot("SetFeatureState");
		_virtkeybd_on = enable;
		showVirtualKeyboard(enable);
		break;
	case kFeatureCursorPalette:
		_use_mouse_palette = enable;
		if (!enable)
			disableCursorPalette();
		break;
	default:
		break;
	}
}

bool OSystem_Android::getFeatureState(Feature f) {
	switch (f) {
	case kFeatureFullscreenMode:
		return _fullscreen;
	case kFeatureAspectRatioCorrection:
		return _ar_correction;
	case kFeatureVirtualKeyboard:
		return _virtkeybd_on;
	case kFeatureCursorPalette:
		return _use_mouse_palette;
	default:
		return false;
	}
}

uint32 OSystem_Android::getMillis() {
	timeval curTime;

	gettimeofday(&curTime, 0);

	return (uint32)(((curTime.tv_sec - _startTime.tv_sec) * 1000) +
			((curTime.tv_usec - _startTime.tv_usec) / 1000));
}

void OSystem_Android::delayMillis(uint msecs) {
	usleep(msecs * 1000);
}

OSystem::MutexRef OSystem_Android::createMutex() {
	pthread_mutexattr_t attr;

	pthread_mutexattr_init(&attr);
	pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);

	pthread_mutex_t *mutex = new pthread_mutex_t;

	if (pthread_mutex_init(mutex, &attr) != 0) {
		warning("pthread_mutex_init() failed");

		delete mutex;

		return 0;
	}

	return (MutexRef)mutex;
}

void OSystem_Android::lockMutex(MutexRef mutex) {
	if (pthread_mutex_lock((pthread_mutex_t *)mutex) != 0)
		warning("pthread_mutex_lock() failed");
}

void OSystem_Android::unlockMutex(MutexRef mutex) {
	if (pthread_mutex_unlock((pthread_mutex_t *)mutex) != 0)
		warning("pthread_mutex_unlock() failed");
}

void OSystem_Android::deleteMutex(MutexRef mutex) {
	pthread_mutex_t *m = (pthread_mutex_t *)mutex;

	if (pthread_mutex_destroy(m) != 0)
		warning("pthread_mutex_destroy() failed");
	else
		delete m;
}

void OSystem_Android::quit() {
	ENTER();

	JNI::setReadyForEvents(false);

	_audio_thread_exit = true;
	pthread_join(_audio_thread, 0);

	_timer_thread_exit = true;
	pthread_join(_timer_thread, 0);

	delete _game_texture;
	delete _overlay_texture;
	delete _mouse_texture_palette;
	delete _mouse_texture_rgb;

	deinitSurface();
}

void OSystem_Android::setWindowCaption(const char *caption) {
	ENTER("%s", caption);

	JNI::setWindowCaption(caption);
}

void OSystem_Android::displayMessageOnOSD(const char *msg) {
	ENTER("%s", msg);

	JNI::displayMessageOnOSD(msg);
}

void OSystem_Android::showVirtualKeyboard(bool enable) {
	ENTER("%d", enable);

	JNI::showVirtualKeyboard(enable);
}

Audio::Mixer *OSystem_Android::getMixer() {
	assert(_mixer);
	return _mixer;
}

void OSystem_Android::getTimeAndDate(TimeDate &td) const {
	struct tm tm;
	const time_t curTime = time(0);

	localtime_r(&curTime, &tm);
	td.tm_sec = tm.tm_sec;
	td.tm_min = tm.tm_min;
	td.tm_hour = tm.tm_hour;
	td.tm_mday = tm.tm_mday;
	td.tm_mon = tm.tm_mon;
	td.tm_year = tm.tm_year;
	td.tm_wday = tm.tm_wday;
}

void OSystem_Android::convertTimeDateToTm(TimeDate &td, struct tm &tm){
	tm.tm_sec = td.tm_sec;
	tm.tm_min = td.tm_min;
	tm.tm_hour = td.tm_hour;
	tm.tm_mday = td.tm_mday;
	tm.tm_mon = td.tm_mon;
	tm.tm_year = td.tm_year;
	tm.tm_wday = td.tm_wday;

}

void OSystem_Android::addSysArchivesToSearchSet(Common::SearchSet &s,
												int priority) {
	ENTER("");

	JNI::addSysArchivesToSearchSet(s, priority);
}

void OSystem_Android::logMessage(LogMessageType::Type type,
									const char *message) {
	switch (type) {
	case LogMessageType::kInfo:
		__android_log_write(ANDROID_LOG_INFO, android_log_tag, message);
		break;

	case LogMessageType::kDebug:
		__android_log_write(ANDROID_LOG_DEBUG, android_log_tag, message);
		break;

	case LogMessageType::kWarning:
		__android_log_write(ANDROID_LOG_WARN, android_log_tag, message);
		break;

	case LogMessageType::kError:
		__android_log_write(ANDROID_LOG_ERROR, android_log_tag, message);
		break;
	}
}

Common::String OSystem_Android::getSystemLanguage() const {
	return Common::String::format("%s_%s",
							getSystemProperty("persist.sys.language").c_str(),
							getSystemProperty("persist.sys.country").c_str());
}

Common::String OSystem_Android::getSystemProperty(const char *name) const {
	char value[PROP_VALUE_MAX];

	int len = __system_property_get(name, value);

	return Common::String(value, len);
}




void OSystem_Android::loadSave1File(){
	AbstractFSNode* _fileSave=_fsFactory->makeFileNodePath(filenameSaveFile1);
	if(!_fileSave->exists()){
		debugdot("le fichier suivant n'existe pas : %s",filenameSaveFile1);
		idLvlToLoad=1;
	}
	else {
		Common::SeekableReadStream* stream=_fileSave->createReadStream();
		
		debugdot("lecture du fichier %s ",filenameSaveFile1);
		Common::String lvl=stream->readLine();
	
		debugdot("**OSystem_Android : loadSaveFile : lvl = %s",lvl.c_str());
		sscanf(lvl.c_str(), "%d", &idLvlToLoad);
	}
	if(idLvlToLoad>1){
		LOGI("idLvlToLoad>1 : true");
		bSimulation=true;
	}
	delete _fileSave;
}



void OSystem_Android::writeSave1File(){

	Common::DumpFile fileSave;
	
	if (!fileSave.open(filenameSaveFile1)) {
		debugdot("WriteSaveFile : ouverture du fichier %s impossible",filenameSaveFile1);
	}
	else {
		debugdot("écriture du fichier : %s",filenameSaveFile1);
		char lvl[2];
		sprintf(lvl, "%d", idLvlCourant);
		fileSave.write(lvl,2);
		fileSave.close();
    	}


}

void OSystem_Android::loadSettingsFile(){
	char* filename;
	
	AbstractFSNode* _fileSettings=_fsFactory->makeFileNodePath(filenameSettings);
	if(!_fileSettings->exists()){
		
		debugdot("le fichier suivant n'existe pas : %s",filenameSettings);
		menu_sound=256;
		sfx_sound=256;
		filter_linear=false;
	}
	else {	
		
		debugdot("écriture du fichier : %s",filenameSettings);
		
		
		Common::SeekableReadStream* stream=_fileSettings->createReadStream();
		Common::String setting;
		setting=stream->readLine();
		//debugdot("**OSystem_Android : loadSettingsFile : lvl = %s",lvl.c_str());
		sscanf(setting.c_str(), "%d", &menu_sound);
		setting=stream->readLine();
		//debugdot("**OSystem_Android : loadSettingsFile : lvl = %s",lvl.c_str());
		sscanf(setting.c_str(), "%d", &sfx_sound);
	
		setting=stream->readLine();
		if(strcmp(setting.c_str(),"true")==0)
				filter_linear=true;
		else filter_linear=false;
		
		
		JNI::setSettings(menu_sound,sfx_sound, filter_linear);	
	}
	
	delete _fileSettings;
}

void OSystem_Android::loadSave2File(){
	char* filename;
	if(currentGame==2)
		filename=filenameSaveFile2;
	if(currentGame==3){
		filename=filenameSaveFile3;
	}
	debugdot("filename = %s,filenameSaveFile = %s",filename,filenameSaveFile3);
	AbstractFSNode* _fileSave=_fsFactory->makeFileNodePath(filename);
	if(!_fileSave->exists()){
		numSlot=20;
		debugdot("le fichier suivant n'existe pas : %s",filename);
	}
	else {	
		Common::SeekableReadStream* stream=_fileSave->createReadStream();
		
		debugdot("lecture du fichier : %s",filename);
		bool lineimpair=false;
		char * buffer=(char*)malloc(28*sizeof(char));
		unsigned int i=0;
		
		Common::String slotToLoad;
		slotToLoad=stream->readLine();
		debugdot("**OSystem_Android : load num slot  = %s",slotToLoad.c_str());
		sscanf(slotToLoad.c_str(), "%d", &numSlot);

		while( !stream->eos() ){
			if(!lineimpair){
				debugdot("i=%d",i);
				buffer=stream->readLine().nc_str();
				debugdot("Empty buffer = %s",buffer);
				saveLoadView.Slot[i].empty = buffer[0] != '0';
				debugdot("empty = %d",saveLoadView.Slot[i].empty);
			}else {
				
				stream->readLine(buffer,27);
				for(unsigned int j=0;j<=26;++j){
					
					if(buffer[j]== 0x0A /*|| buffer[j]==13*/){
						debugdot("Hey == 0x0A!!!!!!!");
						buffer[j]=(char)0;
					}
					debugdot("buffer[%d]=%c  == %d",j,buffer[j],buffer[j]);
					
				}
				saveLoadView.Slot[i].dateLastSave=(char*)malloc(29*sizeof(char));
				
				strcpy(saveLoadView.Slot[i].dateLastSave,buffer);
				//strcpy(Slot[i].dateLastSave,buffer);
				debugdot("dateLastSave = %s",saveLoadView.Slot[i].dateLastSave);
				
				
				/*Slot[i].dateLastSave=(char*)malloc(30*sizeof(char));
				stream->readLine(Slot[i].dateLastSave,30);
				for(unsigned int j=0;j!=30;++j){
					
					if(Slot[i].dateLastSave[j]== 0x0A || Slot[i].dateLastSave[j]==13){
						debugdot("Hey == 0x0A!!!!!!!");
						Slot[i].dateLastSave[j]='\0';
						debugdot("dateLastSave = %s ",Slot[i].dateLastSave);
					}
					debugdot("Slot[%d].dateLastSave[%d]=%c  == %d",i,j,Slot[i].dateLastSave[j],Slot[i].dateLastSave[j]);
				}*/
				debugdot("i=%d",i);
				i++;
			}
			lineimpair=!lineimpair;
			
		}
				
	}
	if(numSlot==20){
			numSlot=0;
			bSimu=false;
		}	
		else{ 
			bSimu=true;
			debugdot("SIIIIIIMMMMUUUUUUU");
			
		}
	
	delete _fileSave;
}

void OSystem_Android::writeSettingsFile(){
	char* filename;
	int number;
	Common::DumpFile fileSettings;
	
	if (!fileSettings.open(filenameSettings)) {
		debugdot("WriteSettingsFile : ouverture du fichier %s impossible",filenameSettings);
	}
	else {
		debugdot("écriture du fichier : %s",filenameSettings);
		char c_music[3];
			sprintf(c_music,"%d\r\n",menu_sound);
			
			debugdot("ceci est c_music %s avec un retour a la ligne",c_music);
			fileSettings.write(c_music,4);
		char c_sfx[3];
			sprintf(c_sfx,"%d\r\n",sfx_sound);
			
			debugdot("ceci est c_sfx %s avec un retour a la ligne",c_sfx);
			fileSettings.write(c_sfx,4);
		char c_filter[6];
			if(filter_linear)
				sprintf(c_filter,"%s\r\n","true");
			else sprintf(c_filter,"%s\r\n","false");
			debugdot("ceci est c_filter %s avec un retour a la ligne",c_filter);
			fileSettings.write(c_filter,7);
		
		fileSettings.close();
    	}


}

void OSystem_Android::writeSave2File(){
	char* filename;
	int number;
	if(currentGame==2){
		filename=filenameSaveFile2;
		number=15;
	}
	if(currentGame==3){
		filename=filenameSaveFile3;
		number=7;
	}
	Common::DumpFile fileSave;
	debugdot("%s,%s",filename,filenameSaveFile3);
	if (!fileSave.open(filename)) {
		debugdot("WriteSaveFile : ouverture du fichier %s impossible",filename);
	}
	else {
		debugdot("écriture du fichier : %s",filename);
		char c_numSlot[3];
			sprintf(c_numSlot,"%d\r\n",numSlot);
			
			debugdot("ceci est c_numSlot %s avec un retour a la ligne",c_numSlot);
			fileSave.write(c_numSlot,3);
		for(unsigned int i=0;i<number;++i){
			char emptybool[3];
			sprintf(emptybool,"%d\r\n",saveLoadView.Slot[i].empty);
			debugdot("i=%d",i);
			debugdot("ceci est empty bool %s avec un retour a la ligne",emptybool);
			fileSave.write(emptybool,3);
			fileSave.write(saveLoadView.Slot[i].dateLastSave,strlen(saveLoadView.Slot[i].dateLastSave));
			debugdot("ceci estdatelastsave %s avec un retour a la ligne",saveLoadView.Slot[i].dateLastSave);
			fileSave.write("\r\n",2);
		}
		fileSave.close();
    	}


}


void OSystem_Android::loadBitmapFontFile(){
	AbstractFSNode* _fileBmpFont=_fsFactory->makeFileNodePath(filenameBitmapFont_black);
	if(!_fileBmpFont->exists()){
		debugdot("le fichier suivant n'existe pas : %s",filenameBitmapFont_black);
		
	}
	else {
		debugdot("lecture du fichier %s ",filenameBitmapFont_black);
		_bmpFont_black.ParseFont(_fileBmpFont->createReadStream());
			
	}

	if(currentGame!=2){
		_fileBmpFont=_fsFactory->makeFileNodePath(filenameBitmapFont_white);
		if(!_fileBmpFont->exists()){
			debugdot("le fichier suivant n'existe pas : %s",filenameBitmapFont_white);
		
		}
		else {
			debugdot("lecture du fichier %s ",filenameBitmapFont_white);
			_bmpFont_white.ParseFont(_fileBmpFont->createReadStream());
			
		}

		_fileBmpFont=_fsFactory->makeFileNodePath(filenameBmpFont_white);
		if(!_fileBmpFont->exists())
			debugdot("erreur le fichier %s n'existe pas ", filenameBmpFont_white);
		else {
			debugdot("lecture du fichier %s ",filenameBmpFont_white);
			_bmpFont_white.loadBmp(_fileBmpFont->createReadStream());	
	
		}
	}
	
	


	_fileBmpFont=_fsFactory->makeFileNodePath(filenameBmpFont_black);
	if(!_fileBmpFont->exists())
		debugdot("erreur le fichier %s n'existe pas ", filenameBmpFont_black);
	else {
		debugdot("lecture du fichier %s ",filenameBmpFont_black);
		_bmpFont_black.loadBmp(_fileBmpFont->createReadStream());	
	
	}
	
	

	
	delete _fileBmpFont;
	//loadXMLFile();
}

/*void OSystem_Android::loadXMLFile(){
	AbstractFSNode* _fileXML=_fsFactory->makeFileNodePath(filenameXML);
	if(!_fileXML->exists()){
		debugdot("le fichier suivant n'existe pas : %s",filenameXML);
		
	}
	else {
		debugdot("lecture du fichier %s ",filenameXML);
		hintView.parseXML(_fileXML->createReadStream());
			
	}
	delete _fileXML;
}*/






#ifdef DYNAMIC_MODULES
void AndroidPluginProvider::addCustomDirectories(Common::FSList &dirs) const {
	((OSystem_Android *)g_system)->addPluginDirectories(dirs);
}
#endif

#endif
