package com.dotemu.gobliiins;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;

import com.dotemu.gobliiins.GamePackage;
import com.dotemu.googleLicensing.APKExpansionPolicy;
import com.dotemu.googleLicensing.GoogleLicensing;
//import com.dotemu.neogeo.R;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.os.Environment;
import android.os.StatFs;
import android.util.Log;

public class GameInstaller
				extends AsyncTask<String, String, String>
{
	public final static int GAME_INSTALLER_ERROR_DOWNLOAD 	= 1;
	public final static int GAME_INSTALLER_ERROR_VERIFY		= 2;
	public final static int GAME_INSTALLER_ERROR_UNPACK		= 3;
	public final static int GAME_INSTALLER_ERROR_FINISH		= 4;
	
	protected Context context;
	private GameInstallerProgressListener listener;
	private ArrayList<GamePackage> packages;
	protected GamePackage currentPackage;
	private String gameDirectory = null;
	private String apkExpansionDir = null;
	private boolean bInstallOccured;	
	protected InputStream standAloneIs = null;
	
	public GameInstaller(Context context, GameInstallerProgressListener listener)
	{
		this.context = context;
		this.listener = listener;
		this.packages = new ArrayList<GamePackage>(10);
		this.currentPackage = null;
		this.gameDirectory = Environment.getExternalStorageDirectory()+"/Android/data/"+context.getPackageName()+"/files";
		this.apkExpansionDir = Environment.getExternalStorageDirectory()+"/Android/obb/"+context.getPackageName();
		this.bInstallOccured = false;		
	}

	// Helper method to check if the device is only
    public boolean isOnline() 
    {
    	 ConnectivityManager cm = (ConnectivityManager)context.getSystemService(Context.CONNECTIVITY_SERVICE);
    	 NetworkInfo netInfo = cm.getActiveNetworkInfo();
    	 if (netInfo==null)
    		 return false;
    	 return netInfo.isConnected();
    }
    
	// Static method to check if space available on filesystem
    public boolean isSpaceAvailable(int megaBytes)
    {
    	StatFs statFs = new StatFs(Environment.getExternalStorageDirectory().getAbsolutePath());
    	long nBlocks = statFs.getFreeBlocks();
    	int iBlockSize = statFs.getBlockSize();
    	long lSize = nBlocks*iBlockSize;
    	if (lSize < megaBytes*1024*1024) 
    		return false;
    	return true;
    }
    
	// Method to add a package to the installation list
	public void addPackage(GamePackage gamePackage)
	{
		if ((packages!=null) && (gamePackage!=null))
		{
			packages.add(gamePackage);
		}
	}
	
	public void start()
	{
		// TODO
	}
	
	public void stop()
	{
		// TODO
	}
	
    public final boolean checkSdCard()
    {
    	String externalState = android.os.Environment.getExternalStorageState();
    	if (externalState.equals(android.os.Environment.MEDIA_MOUNTED)) 
    		return true;

    	if ( (externalState.equals(android.os.Environment.MEDIA_UNMOUNTED)) || (externalState.equals(android.os.Environment.MEDIA_REMOVED)))
    	{
    		// R.string.UNMOUNT_MEDIA
    		if (listener!=null)
    			listener.onSdCardError(externalState);
    		return false;
        }
    	
    	if (externalState.equals(android.os.Environment.MEDIA_BAD_REMOVAL)) 
    	{
    		// R.string.MEDIA_PROBLEM
    		if (listener!=null)
    			listener.onSdCardError(externalState);
    		return false;
        }
    	
    	if (externalState.equals(android.os.Environment.MEDIA_CHECKING)) 
    	{
    		// R.string.MEDIA_NOT_READY
    		if (listener!=null)
    			listener.onSdCardError(externalState);
    		return false;
        }
    	
    	if (externalState.equals(android.os.Environment.MEDIA_SHARED)) 
    	{
    		// R.string.UNMOUNT_MEDIA
    		if (listener!=null)
    			listener.onSdCardError(externalState);
    		return false;
    	}    	
    	else //if (android.os.Environment.getExternalStorageState().equals(android.os.Environment.MEDIA_CHECKING)) 
    	{
    		// R.string.NO_SPACE (by default)
    		if (listener!=null)
    			listener.onSdCardError(externalState);
    		return false;
        }
    }
	
	public boolean checkCurrentPackage()
	{
		// Check if everything is setup correctly, return false if not
		if (currentPackage==null)
			return false;
		
		// Check if marker file exists and is a file
		File fMarker = new File(gameDirectory, currentPackage.getMarker());
		if ((fMarker!=null) && (fMarker.exists()) && (fMarker.isFile()))
			return true;
		return false;
	}
	
	public boolean downloadCurrentPackage()
	{	
		if(currentPackage.getUrl().equals("APK")) {
			currentPackage.setFullPath(currentPackage.getFile());
			return true;
		}
		
		if(currentPackage.getUrl().equals("STAND_ALONE")) {
			currentPackage.setFullPath(currentPackage.getFile());// Check if ZIP file exists
			return true;
		}
		
		if(GoogleLicensing.enableGoogleLVL) {
			
			if(currentPackage.getUrl().equals("Main Expansion")) {
				APKExpansionPolicy lvlPolicy = GoogleLicensing.GetApkPolicy();
			//	Log.e("dotemu",""+lvlPolicy.getExpansionURLCount());
				if(lvlPolicy.getExpansionURLCount() < 1) return false;
				
				String packageURL = (lvlPolicy.getExpansionURL(APKExpansionPolicy.MAIN_FILE_URL_INDEX));
				String packageFile = lvlPolicy.getExpansionFileName(APKExpansionPolicy.MAIN_FILE_URL_INDEX);
				int packageSize = (int) lvlPolicy.getExpansionFileSize(APKExpansionPolicy.MAIN_FILE_URL_INDEX);
				
				// if we receive empty informations from Google Play servers then it would mean that we've already downloaded the expansion files!
				/*if ((packageURL == null)|| (packageSize <= 0)) {
					// check if .OBB files were already downloaded
					String fileName = String.format("main.8.%s.obb", context.getPackageName()); // NEED TO CHANGE VERSION CODE IF NEW FILES UPLOADED
					File file = new File(apkExpansionDir + '/' + fileName);
					
					if (file.exists() && file.isFile()) {
						packageFile = fileName;
						packageSize = (int) file.length();
					} else {
						Log.e(GoogleLicensing.PL_TAG, "Can not find Google Play OBB files... Exit!");
						return false;
					}
				}*/
				currentPackage.setURL(packageURL);
				currentPackage.setFullPath(apkExpansionDir + "/" + packageFile);			
			} 
			else if(currentPackage.getUrl().equals("Patch Expansion")) {
				APKExpansionPolicy lvlPolicy = GoogleLicensing.GetApkPolicy();
				if(lvlPolicy.getExpansionURLCount() < 2) return false;
				currentPackage.setURL(lvlPolicy.getExpansionURL(APKExpansionPolicy.PATCH_FILE_URL_INDEX));
				currentPackage.setFullPath(apkExpansionDir + "/" + lvlPolicy.getExpansionFileName(APKExpansionPolicy.PATCH_FILE_URL_INDEX));
			} else {
				currentPackage.setFullPath(gameDirectory+"/"+currentPackage.getFile());
			}
		} else {
			currentPackage.setFullPath(gameDirectory+"/"+currentPackage.getFile());
		}
		
    	HttpParams httpParameters = new BasicHttpParams();
    	int timeoutConnection = 20000;
    	int timeoutSocket = 20000;
    	// Set the timeout in milliseconds until a connection is established.
    	// The default value is zero, that means the timeout is not used. 
    	HttpConnectionParams.setConnectionTimeout(httpParameters, timeoutConnection);
    	// Set the default socket timeout (SO_TIMEOUT) 
    	// in milliseconds which is the timeout for waiting for data.
    	HttpConnectionParams.setSoTimeout(httpParameters, timeoutSocket);
    	
    	// AndroidHttpClient is not allowed to be used from the main thread
        final HttpClient client = new DefaultHttpClient(httpParameters);// .newInstance("Android");
        final HttpGet getRequest = new HttpGet(currentPackage.getUrl());
        File zipFile = null;
        
    	// Check if ZIP file exists
        zipFile = new File(currentPackage.getFullPath());
        if (zipFile.exists()==false)
        {
        	try
        	{
        		new File(zipFile.getParent()).mkdirs();
        		zipFile.createNewFile();
        	}
        	catch (IOException e)
        	{
                //Log.e("FileDownloader", "Error cannot while creating ZIP file at " + gameDirectory.getAbsolutePath());
        		e.printStackTrace();
                return false;
        	}
        }
        try 
        {
            HttpResponse response = client.execute(getRequest);
            final int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode != HttpStatus.SC_OK) 
            {
                Log.e("FileDownloader", "Error " + statusCode + " while retrieving " + currentPackage.getUrl());
                return false;
            }
            final HttpEntity entity = response.getEntity();
            if (entity != null) 
            {
                InputStream inputStream = null;
                try 
                {
                	long readBytes = 0;
                	long totalBytes = entity.getContentLength();
                   	if (zipFile.length()!=totalBytes)
                   	{
                        inputStream = entity.getContent();
                        BufferedInputStream bis = new BufferedInputStream(inputStream);
	                    byte[] buffer = new byte[262144*2];
	                    FileOutputStream fos = new FileOutputStream(zipFile);
	                    int bytesRead = 0;
	                    do
	                    {
	                    	bytesRead = bis.read(buffer, 0, buffer.length);
	                    	if (bytesRead > 0)
	                    	{
	                    		readBytes += bytesRead;
	                    		fos.write(buffer, 0, bytesRead);
	                    		if (listener!=null)
	                    			listener.onPackageDownload((int)readBytes);
	                    	}	              
	                    	
	            			// Is task cancelled ?
	            			if (isCancelled())
	            				return false;

	                    } while (bytesRead != -1);
	                    bis.close();
	                    fos.flush();
	                    fos.close();
                   	}
                }
                finally 
                {
                    if (inputStream != null) 
                    {
                        inputStream.close();
                        entity.consumeContent();
                    }
                }
                System.gc();
            }
        }
        catch (IOException e) 
        {
            getRequest.abort();
            //LaunchActivity.updateInfo(R.string.DOWNLOAD_ERROR);
            Log.e("FileDownloader", "I/O error while retrieving file from " + currentPackage.getUrl(), e);
        } 
        catch (IllegalStateException e) 
        {
            getRequest.abort();
            //LaunchActivity.updateInfo(R.string.DOWNLOAD_ERROR);
            Log.e("FileDownloader", "Incorrect URL: " + currentPackage.getUrl());
        } 
        catch (Exception e) 
        {
            getRequest.abort();
            //LaunchActivity.updateInfo(R.string.DOWNLOAD_ERROR);
            Log.e("FileDownloader", "Error while retrieving file from " + currentPackage.getUrl(), e);
        } 
		return true;
	}
	
	public boolean unpackCurrentPackage()
	{
		if (currentPackage==null)
			return false;

		// Do we need to unpack ?
		if (currentPackage.getUnpack()==false)
			return true;
		
		String filePath = currentPackage.getFullPath();
		
		InputStream is;
		BufferedInputStream bis;
		ZipInputStream zis;
		try
		{
			// Notify listener we are unpacking !
			if (listener!=null)
				listener.onPackageUnpack("", 0);

			if(currentPackage.getUrl().equals("STAND_ALONE"))
			{
				ZipInputStream standAloneZis;
				
				try 
				{
					// Retrieve number of files inside ZIP
					int nEntries = 0;
					standAloneZis = new ZipInputStream(new BufferedInputStream(standAloneIs));
					while (standAloneZis.getNextEntry()!=null)
						nEntries++;
					standAloneZis = null;
					int nProcessedEntries=0;

					// Rebuild the input stream
					standAloneIs.reset();
					standAloneZis = new ZipInputStream(new BufferedInputStream(standAloneIs));
					ZipEntry ze;

					// Create all required directories
					try 
					{
						while ((ze = standAloneZis.getNextEntry()) != null) 
						{
							if (ze.isDirectory())
							{
								nProcessedEntries++;

								// Notify listener we are unpacking !
								String filename = ze.getName();
								if (listener!=null)
									listener.onPackageUnpack(filename, nProcessedEntries*100/nEntries);

								File fDir = new File(gameDirectory+"/"+filename);
								fDir.mkdirs();	                	 
								File noMediaFile = new File(fDir, ".nomedia");
								noMediaFile.createNewFile();

								// Is task cancelled ?
								if (isCancelled())
									return false;
							}	                 	                 
						}
					}
					catch(IOException e) 
					{
						e.printStackTrace();
					}
					
					// Rebuild the input stream
					standAloneIs.reset();
					standAloneZis = new ZipInputStream(new BufferedInputStream(standAloneIs));

					// Now unzip all ZIP entries
					byte[] buffer = new byte[131072*2];
					while ((ze = standAloneZis.getNextEntry()) != null) 
					{
						if (ze.isDirectory())
							continue;
						int count;

						nProcessedEntries++;
						String filename = ze.getName();

						if(currentPackage.getUrl().equals("APK")) {
							if(!filename.startsWith("assets/")) continue;
							else
							{
								filename = filename.substring(7);
								if(filename.startsWith("SLAwards.bundle/")) continue; // SCORELOOP - Hack to allow ScoreLoop SLAwards.bundle in assets folder
								/*if(DotEmuActivity.T_STORE_BUILD)
		                		{
		                			if(filename.startsWith("sdex/")) continue; // Temp hack to avoid DRM folder. TODO: Should implement code to handle folder instead.
		                		}*/
							}
						}

						int lastSeparator = filename.lastIndexOf('/');
						String fName = lastSeparator != -1 ? filename.substring(lastSeparator + 1) : filename;

						if (listener!=null)
							listener.onPackageUnpack(fName, (nProcessedEntries*100)/nEntries);
						//LaunchActivity.updateInfo(fName);

						// Is task cancelled ?
						if (isCancelled())
							return false;

						File fFile = new File(gameDirectory+"/"+filename);

						// Does file exists ?
						if (fFile.exists())
						{
							// If size match, we can guess it's already OK
							// If it does not match, delete the file !
							if (fFile.length()==ze.getSize())
								continue;
							else
								fFile.delete();
						}

						// Create and unpack the new file
						fFile.createNewFile();
						FileOutputStream fout = new FileOutputStream(gameDirectory+"/"+filename);
						BufferedOutputStream bos = new BufferedOutputStream(fout,65536);
						while ((count = standAloneZis.read(buffer)) != -1) 
						{
							bos.write(buffer, 0, count);
						}
						bos.flush();
						fout.flush();
						bos.close();
						fout.close();
						standAloneZis.closeEntry();

						// Is task cancelled ?
						if (isCancelled())
							return false;
					}
					standAloneZis.close();
				}
				catch(IOException e)
				{
					//LaunchActivity.updateInfo("Error while unpacking data");
					e.printStackTrace();
					return false;
				}
			}
			else 
			{
				// Retrieve number of files inside ZIP
				ZipFile zf = new ZipFile(filePath);
				int nEntries = 0;
				Enumeration<? extends ZipEntry> entries = zf.entries();
				while (entries.hasMoreElements())
				{
					entries.nextElement();
					nEntries++;
				}
				int nProcessedEntries=0;

				// Now build the input streams
				is = new FileInputStream(filePath);
				bis = new BufferedInputStream(is);
				zis = new ZipInputStream(bis);          
				ZipEntry ze;

				// Create all required directories
				// Can be skipped here, no directories in this project
				try 
                {
    	             while ((ze = zis.getNextEntry()) != null) 
    	             {
    	                 if (ze.isDirectory())
    	                 {
    		            	 nProcessedEntries++;

    		            	 // Notify listener we are unpacking !
    		            	 String filename = ze.getName();
    		            	 if (listener!=null)
    		            		 listener.onPackageUnpack(filename, nProcessedEntries*100/nEntries);

    		            	 File fDir = new File(gameDirectory+"/"+filename);
    		            	 fDir.mkdirs();	                	 
    		            	 File noMediaFile = new File(fDir, ".nomedia");
    		            	 noMediaFile.createNewFile();

    	            			// Is task cancelled ?
    	            			if (isCancelled())
    	            				return false;
    	                 }	                 	                 
    	             }
                }
                catch(IOException e) 
                {
                	e.printStackTrace();
                }

				is = new FileInputStream(filePath);
				bis = new BufferedInputStream(is);
				zis = new ZipInputStream(bis);          

				// Now unzip all ZIP entries
				byte[] buffer = new byte[131072*2];
				while ((ze = zis.getNextEntry()) != null) 
				{
					if (ze.isDirectory())
						continue;
					int count;

					nProcessedEntries++;
					String filename = ze.getName();

					//Log.i("TAG", "FILENAME 2 = " + filename);

					if(currentPackage.getUrl().equals("APK")) {
						if(!filename.startsWith("assets/")) continue;
	                	else
	                	{
	                		filename = filename.substring(7);
	                		if(filename.startsWith("SLAwards.bundle/")) continue; // SCORELOOP - Hack to allow ScoreLoop SLAwards.bundle in assets folder
	                	/*	if(DotEmuActivity.T_STORE_BUILD)
	                		{
	                			if(filename.startsWith("sdex/")) continue; // Temp hack to avoid DRM folder. TODO: Should implement code to handle folder instead.
	                		}*/
	                	}
					}

					int lastSeparator = filename.lastIndexOf('/');
					String fName = lastSeparator != -1 ? filename.substring(lastSeparator + 1) : filename;

					if (listener!=null)
						listener.onPackageUnpack(fName, (nProcessedEntries*100)/nEntries);
						//LaunchActivity.updateInfo(fName);

						// Is task cancelled ?
						if (isCancelled())
							return false;

					File fFile = new File(gameDirectory+"/"+filename);

					// Does file exists ?
					if (fFile.exists())
					{
						// If size match, we can guess it's already OK
						// If it does not match, delete the file !
						if (fFile.length()==ze.getSize())
							continue;
						else
							fFile.delete();
					}

					// Create and unpack the new file
					fFile.createNewFile();
					FileOutputStream fout = new FileOutputStream(gameDirectory+"/"+filename);
					BufferedOutputStream bos = new BufferedOutputStream(fout,65536);
					while ((count = zis.read(buffer)) != -1) 
					{
						bos.write(buffer, 0, count);
					}
					bos.flush();
					fout.flush();
					bos.close();
					fout.close();
					zis.closeEntry();

					// Is task cancelled ?
							if (isCancelled())
								return false;
				}
				zis.close();
			} 

		}	
		catch(IOException e)
		{
			e.printStackTrace();
			return false;
		}

		// Is task cancelled ?
		if (isCancelled())
			return false;

		// Everything went fine
		return true;
	}
	
	public boolean finishCurrentPackage()
	{
		// Check if marker file exists and is a file
		File fMarker = new File(gameDirectory, currentPackage.getMarker());
		boolean bResult = false;
		try
		{
			bResult = fMarker.createNewFile();
		} 
		catch (Exception e)
		{
			e.printStackTrace();
		}
		if (currentPackage.getUnpack())
		{
			try
			{
				File fZip = new File(gameDirectory, currentPackage.getFile());
				fZip.delete();
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		return bResult;
	}
	
	public boolean verifyCurrentPackage()
	{
		// No valid package, return false
		if (currentPackage==null)
			return false;
		
		// No MD5 verified, so skip MD5 checking
		if ((currentPackage.getMD5()==null) || (currentPackage.getMD5().isEmpty())) 
			return true;
		
		int bufferReadSize = 65536;
		try
		{
			 // Try to open file
			 File file = new File(gameDirectory + "/" + currentPackage.getFile());
			 if ((file==null) || (file.exists()==false))
				 return false;
			 FileInputStream fis = new FileInputStream(file);
			 byte[] fileBuffer = new byte[bufferReadSize];

			 // Create MD5 Hash
			 MessageDigest digest = java.security.MessageDigest.getInstance("MD5");
			 int bytesRead = -1;
			 do
			 {
				 bytesRead = fis.read(fileBuffer, 0, bufferReadSize);
				 if (bytesRead != -1)
					 digest.update(fileBuffer, 0, bytesRead);
			 } while (bytesRead != -1);			 
			 byte messageDigest[] = digest.digest();
		        
			 // Create Hex String
		     StringBuffer hexString = new StringBuffer();
		     for (int i=0; i<messageDigest.length; i++)
		    	 hexString.append(Integer.toHexString(0xFF & messageDigest[i]));
		     String computedHash = hexString.toString();

		     // Compare and take decision
		     if (computedHash.equalsIgnoreCase(currentPackage.getMD5()))
		     {
		    	 return true;
		     }
		 } 
		 catch (Exception e) 
		 {
		        e.printStackTrace();
		        return false;
		 }
		 return false;
	}

	private final void createRequiredDirectories()
	{
		File fDirectory = new File(Environment.getExternalStorageDirectory()+"/Android");
		if (fDirectory.exists()==false)
			fDirectory.mkdir();
		fDirectory = new File(Environment.getExternalStorageDirectory()+"/Android/data");
		if (fDirectory.exists()==false)
			fDirectory.mkdir();
		fDirectory = new File(Environment.getExternalStorageDirectory()+"/Android/data/"+context.getPackageName());
		if (fDirectory.exists()==false)
			fDirectory.mkdir();
		fDirectory = new File(Environment.getExternalStorageDirectory()+"/Android/data/"+context.getPackageName()+"/files");
		if (fDirectory.exists()==false)
			fDirectory.mkdir();
	}
	
	public final boolean checkGameInstallation()
	{
		// Create required directories if needed
		createRequiredDirectories();
		for (GamePackage gp : packages)
		{
			// This is the current package we are processing.
			currentPackage = gp;

			// Check if currentPackage is properly installed using marker.
			if (checkCurrentPackage()==false)
				return false;
		}
		return true;
	}
	
	@Override
	// This is the main installation process
    // Before treating each package, we check if SDCard is okay
	// For each package do the following steps 
	// 1. Download
	// 2. Verify (MD5)
	// 3. Unpack, if needed
	// 4. Finish (mark as done on the filesystem)
	protected String doInBackground(String... params) 
	{
		// Create required directories if needed
		createRequiredDirectories();
		
		// Create .nomedia file at root game directory, once, if needed
		File noMediaFile = new File(gameDirectory, ".nomedia");
		try 
		{
			if (noMediaFile.exists()==false)
				noMediaFile.createNewFile();
		} 
		catch(IOException e) 
		{
			e.printStackTrace();
		}
		
		for (GamePackage gp : packages)
		{
			// This is the current package we are processing.
			// Notify listener
			currentPackage = gp;
			if (listener!=null)
				listener.onPackageBegin(currentPackage.getName(), currentPackage.getSize());
			
			// Is task cancelled ?
			if (isCancelled())
				return null;

			// Check if currentPackage is properly installed using marker.
			// If that's the case, let's skip it !
			if (checkCurrentPackage()==true)
				continue;
				
			// At least one install occured
			bInstallOccured = true;
			
			// Is task cancelled ?
			if (isCancelled())
				return null;

			// If offline, download will fail !
			if(!(LaunchActivity.STAND_ALONE_BUILD))
			{
			//	Log.e("Dotemu","Check onLine if Stand ALONE");
				if (isOnline()==false)
				{
					// Notify listener we are offline
					if (listener!=null)
						listener.onNoInternetError();
					return null;
				}
			}

			// Is task cancelled ?
			if (isCancelled())
				return null;

			// Check free space
			int packageSize = currentPackage.getSize()/1024/1024;
			if (currentPackage.getUnpack()) packageSize *= 2;
			if (isSpaceAvailable(packageSize)==false)
			{
				if (listener!=null)
					listener.onNoFreeSpaceError(packageSize);
				return null;
			}
			
			// Time for download
		//	Log.e("Dotemu","DOWNLOAD");
			if (downloadCurrentPackage()==false)
			{
				// Notify listener download has failed, abort everything !
				if (listener!=null)
					listener.onPackageError(currentPackage.getName(),GAME_INSTALLER_ERROR_DOWNLOAD);
				return null;
			}

			// Is task cancelled ?
			if (isCancelled())
				return null;

			// Time for verify
			
			if (verifyCurrentPackage()==false)
			{
				// Notify listener verify has failed, abort everything !
				if (listener!=null)
					listener.onPackageError(currentPackage.getName(),GAME_INSTALLER_ERROR_VERIFY);
				return null;
			}

			// Is task cancelled ?
			if (isCancelled())
				return null;
		//	Log.e("Dotemu","UNPACK");
			// Time for unpack, if needed
			if (unpackCurrentPackage()==false)
			{
				// Notify listener unpack has failed, abort everything !
				if (listener!=null)
					listener.onPackageError(currentPackage.getName(),GAME_INSTALLER_ERROR_UNPACK);
				return null;
			}

			// Is task cancelled ?
			if (isCancelled())
				return null;

			// Time to finish
			if (finishCurrentPackage()==false)
			{
				// Notify listener finish has failed, abort everything !
				if (listener!=null)
					listener.onPackageError(currentPackage.getName(),GAME_INSTALLER_ERROR_FINISH);
				return null;
			}
		}
		// Finished the job, clear the packages list if we want to reuse the object and run another job
	//	Log.e("Dotemu","UNPACK");
		packages.clear();		
		return "OK";
	}
	
	protected void onPostExecute(String string)
	{
		// Notify the listener we're done
		if (listener!=null)
			listener.onInstallFinished(bInstallOccured);
	}
}
