package com.dotemu.gobliiins;


import com.dotemu.gobliiins.R;

import android.content.Context;
import android.media.MediaPlayer;
import android.util.Log;


public class MusicManager {
	private static final String TAG = "MusicManager";
	public static final int MUSIC_PREVIOUS = -1;
	public static final int MUSIC_GOB1 = 1;
	public static final int MUSIC_GOB2 = 2;
	public static final int MUSIC_GOB3 = 3;

	private static int currentMusic = -1;
	private static int previousMusic = -1;
	
	static MediaPlayer mp = null;

	public static void start(Context context, int music) {
		start(context, music, false);
	}

	public static void start(Context context, int music, boolean force) {
		if (!force && currentMusic > -1) {
			// already playing some music and not forced to change
			return;
		}
		if (music == MUSIC_PREVIOUS) {
			//Log.d(TAG, "Using previous music [" + previousMusic + "]");
			music = previousMusic;
		}
		if (currentMusic == music) {
			// already playing this music
			return;
		}
		if (currentMusic != -1) {
			previousMusic = currentMusic;
			//Log.d(TAG, "Previous music was [" + previousMusic + "]");
			// playing some other music, pause it and change
			pause();
		}
		currentMusic = music;
		//Log.d(TAG, "Current music is now [" + currentMusic + "]");
		
		if (mp == null || music != previousMusic) 
		{
			if (music == MUSIC_GOB1) {
				mp = MediaPlayer.create(context, R.raw.gob1);
			} else if (music == MUSIC_GOB2) {
				mp = MediaPlayer.create(context, R.raw.gob2);
			} else if (music == MUSIC_GOB3) {
				mp = MediaPlayer.create(context, R.raw.gob3);
			} else {
				Log.e(TAG, "unsupported music number - " + music);
				return;
			}
		}
		
		if (mp == null) 
			Log.e(TAG, "player was not created successfully");
		else 
		{
			try {
				mp.setLooping(true);
				mp.start();
			} catch (Exception e) {
				Log.e(TAG, e.getMessage(), e);
			}
		}
	}

	public static void pause() 
	{
		//Log.d(TAG, "music manager pause");
		if (mp!=null){
			if (mp.isPlaying()) {
				//Log.d(TAG, "if playing music manager pause");
				mp.pause();
				//Log.d(TAG, " done music manager pause");
			}
		}
		
		// previousMusic should always be something valid
		if (currentMusic != -1) {
			previousMusic = currentMusic;
			//Log.d(TAG, "Previous music was [" + previousMusic + "]");
		}
		currentMusic = -1;
		//Log.d(TAG, "Current music is now [" + currentMusic + "]");
	}
	
	public static void setVolume(float volume)
	{
		if(mp != null)
			mp.setVolume(volume, volume);
	}

	public static void release() {
		//Log.d(TAG, "Releasing media players");
		try {
			if (mp != null) {
				if (mp.isPlaying()) {
					mp.stop();
				}
				mp.release();
				mp = null;
			}
		} catch (Exception e) {
			Log.e(TAG, e.getMessage(), e);
		}
		
		if (currentMusic != -1) {
			previousMusic = currentMusic;
			//Log.d(TAG, "Previous music was [" + previousMusic + "]");
		}
		currentMusic = -1;
		//Log.d(TAG, "Current music is now [" + currentMusic + "]");
	}
	
}
