package com.dotemu.gobliiins;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ComponentName;
import android.content.DialogInterface;
import android.content.Intent;
import android.media.AudioManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.SurfaceView;
import android.view.SurfaceHolder;
import android.view.MotionEvent;
import android.view.inputmethod.InputMethodManager;
import android.widget.Toast;

import java.io.File;
import java.net.URLEncoder;
import java.util.Locale;

import com.dotemu.gobliiins.R;

public class ScummVMActivity extends Activity {

	/* Establish whether the hover events are available */
	private static boolean _hoverAvailable;
	private static final int ID_ACTIVITY_SETTINGS=1;
	private static final int ID_ACTIVITY_HINTS=2;
	private int menu_sound=256;
	private int sfx_sound=256;
	private boolean checked=false;

	static {
		try {
			MouseHelper.checkHoverAvailable(); // this throws exception if we're on too old version
			_hoverAvailable = true;
		} catch (Throwable t) {
			_hoverAvailable = false;
		}
	}

	private class MyScummVM extends ScummVM {
		private boolean usingSmallScreen() {
			// Multiple screen sizes came in with Android 1.6.  Have
			// to use reflection in order to continue supporting 1.5
			// devices :(
			DisplayMetrics metrics = new DisplayMetrics();
			getWindowManager().getDefaultDisplay().getMetrics(metrics);

			try {
				// This 'density' term is very confusing.
				int DENSITY_LOW = metrics.getClass().getField("DENSITY_LOW").getInt(null);
				int densityDpi = metrics.getClass().getField("densityDpi").getInt(metrics);
				return densityDpi <= DENSITY_LOW;
			} catch (Exception e) {
				return false;
			}
		}

		public MyScummVM(SurfaceHolder holder) {
			super(ScummVMActivity.this.getAssets(), holder);

			// Enable ScummVM zoning on 'small' screens.
			// FIXME make this optional for the user
			// disabled for now since it crops too much
			//enableZoning(usingSmallScreen());
		}

		@Override
		protected void getDPI(float[] values) {
			DisplayMetrics metrics = new DisplayMetrics();
			getWindowManager().getDefaultDisplay().getMetrics(metrics);

			values[0] = metrics.xdpi;
			values[1] = metrics.ydpi;
		}

		@Override
		protected void displayMessageOnOSD(String msg) {
			Log.i(LOG_TAG, "OSD: " + msg);
			Toast.makeText(ScummVMActivity.this, msg, Toast.LENGTH_LONG).show();
		}

		@Override
		protected void setWindowCaption(final String caption) {
			runOnUiThread(new Runnable() {
					public void run() {
						setTitle(caption);
					}
				});
		}

		@Override
		protected String[] getPluginDirectories() {
			String[] dirs = new String[1];
			dirs[0] = ScummVMApplication.getLastCacheDir().getPath();
			return dirs;
		}

		@Override
		protected void showVirtualKeyboard(final boolean enable) {
			//Log.i("dotemu","showVirtualKeyboard");
			runOnUiThread(new Runnable() {
					public void run() {
						showKeyboard(enable);
					}
				});
		}
		
		@Override
		protected void showHintMenu(final int idLvl) {
			//Log.i("dotemu","showHtmlBrowser");
		
			Intent t = new Intent(ScummVMActivity.this, HintViewActivity.class);
			t.putExtra("idLvl",idLvl);
		
			t.putExtra("game", _game_launch);
			startActivityForResult(t,ID_ACTIVITY_HINTS);
			
		}
		
		@Override
		protected void showSettingsMenu() {
			//Log.i("dotemu","showSettingsMenu");
		
			Intent t = new Intent(ScummVMActivity.this, SettingsMenuActivity.class);
		
	        t.putExtra("menu_sound",menu_sound);
	        t.putExtra("sfx_sound",sfx_sound);  
	        t.putExtra("Checked",checked);
			t.putExtra("game", _game_launch);
			t.putExtra("in_game", true);
			startActivityForResult(t,ID_ACTIVITY_SETTINGS);
			
			
		}
		
		

		@Override
		protected String[] getSysArchives() {
			return new String[0];
		}

		@Override
		protected void setSettings(int musicSetting, int sfxSetting, boolean filterSetting) {
			//Log.i("dotemu","setSettings");
			
			menu_sound=musicSetting;
			sfx_sound=sfxSetting;
			checked=filterSetting;
			
		}

	}

	private MyScummVM _scummvm;
	private ScummVMEvents _events;
	private MouseHelper _mouseHelper;
	private Thread _scummvm_thread;
	private String _game_launch;
	private String _path_game;
	private String Language;

	
	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {     
		  super.onActivityResult(requestCode, resultCode, data); 
		  switch(requestCode) { 
		    case ID_ACTIVITY_SETTINGS : { 
		      if (resultCode == RESULT_OK) { 
		    	  Bundle b=data.getExtras();
		          menu_sound=b.getInt("music");
		          sfx_sound=b.getInt("sfx");
		          checked=b.getBoolean("Checked");
		    	 // Log.i("dotemu","Hey onActivtyResul musix ="+menu_sound+    "sfx="+sfx_sound);
	
			      _scummvm.setSoundSettings(menu_sound,sfx_sound,checked);
			      // TODO Update your TextView.
			      } 
		      break; 
		    } 
		  } 
		}
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setVolumeControlStream(AudioManager.STREAM_MUSIC);

		setContentView(R.layout.main);
		takeKeyEvents(true);
		Intent intent= getIntent();
		if(intent != null){
			_game_launch=intent.getStringExtra("game");
		}
		//Log.i("ScummVmActivity",_game_launch);
		_path_game="default";
		// This is a common enough error that we should warn about it
		// explicitly.
		if (!Environment.getExternalStorageDirectory().canRead()) {
			new AlertDialog.Builder(this)
				.setTitle(R.string.no_sdcard_title)
				.setIcon(android.R.drawable.ic_dialog_alert)
				.setMessage(R.string.no_sdcard)
				.setNegativeButton(R.string.quit,
									new DialogInterface.OnClickListener() {
										public void onClick(DialogInterface dialog,
															int which) {
											finish();
										}
									})
				.show();

			return;
		}

		SurfaceView main_surface = (SurfaceView)findViewById(R.id.main_surface);

		main_surface.requestFocus();

		getFilesDir().mkdirs();

		// Store savegames on external storage if we can, which means they're
		// world-readable and don't get deleted on uninstall.
		String savePath = Environment.getExternalStorageDirectory() + "/Android/data/com.dotemu.gobliiins/files/ScummVM/Saves/";
		Language=Locale.getDefault().getLanguage();
		//Log.i("dotemu","Language = "+ Language);
		
		if(!"fr".equals(Language) && !"en".equals(Language)/* && !"ru".equals(Language) && !"pl".equals(Language)*/)
			Language="en";
		File saveDir = new File(savePath);
		saveDir.mkdirs();
		if (!saveDir.isDirectory()) {
			// If it doesn't work, resort to the internal app path.
			savePath = getDir("saves", MODE_WORLD_READABLE).getPath();
		}
		if("gob1".equals(_game_launch)) {
			_path_game=Environment.getExternalStorageDirectory().getPath()+"/Android/data/com.dotemu.gobliiins/files/Gob/Gob1"; //trunk/dists/iphone/gob1
		}
		if("gob2".equals(_game_launch)){
			_path_game=Environment.getExternalStorageDirectory().getPath()+"/Android/data/com.dotemu.gobliiins/files/Gob/Gob2"; //trunk/dists/iphone/gob2
		}
		if("gob3".equals(_game_launch)){
			_path_game=Environment.getExternalStorageDirectory().getPath()+"/Android/data/com.dotemu.gobliiins/files/Gob/Gob3"; // trunk/dists/iphone/gob3
		}
	//	Log.e("scummVMActivity",_path_game);
		// Start ScummVM
		_scummvm = new MyScummVM(main_surface.getHolder());
	//	Log.e("dotemu","SetArgs");
	//	Log.i("dotemu",getFileStreamPath("scummvmrc").getPath());
		_scummvm.setArgs(new String[] {
			"ScummVM",
			"--config=" + getFileStreamPath("scummvmrc").getPath(),
			"--language="+ Language ,
			//"-gfilter", 
			"--path=" + _path_game,"-F",
			_game_launch 
		});
		//Log.d(ScummVM.LOG_TAG, "Hover available: " + _hoverAvailable);
		if (_hoverAvailable) {
			_mouseHelper = new MouseHelper(_scummvm);
			_mouseHelper.attach(main_surface);
		}

		_events = new ScummVMEvents(this, _scummvm, _mouseHelper);

		main_surface.setOnKeyListener(_events);
		main_surface.setOnTouchListener(_events);
	//	Log.i("scummVmActivity","lancement du thread");
		_scummvm_thread = new Thread(_scummvm, "ScummVM");
		_scummvm_thread.start();
	}

	@Override
	public void onStart() {
//		Log.d(ScummVM.LOG_TAG, "onStart");

		super.onStart();
	}
	
	private boolean focused =false;
	
	@Override
	public void onWindowFocusChanged(boolean focus)
	{
		super.onWindowFocusChanged(focus);
		SurfaceView s = (SurfaceView)findViewById(R.id.main_surface);
//		Log.d(ScummVM.LOG_TAG, "onWindowFocusChanged focus = "+focus);
		
		if ((focused==true) && (focus==false) && (_scummvm!=null))
		{
			_scummvm.hide(s.getHolder());
			focused = false;
		}
		else if ((focused==false) && (focus==true) && (_scummvm!=null))
		{
			_scummvm.show(s.getHolder());
			focused = true;
		}
	}

	@Override
	public void onResume() {
//		Log.d(ScummVM.LOG_TAG, "onResume");

		super.onResume();

		if (_scummvm != null) 
		{
			_scummvm.setPause(false);
			
			// DotEmu hack to call surfaceChanged when resuming game after going to sleep via power button
			// The surface will be created a second time via the true SurfaceChanged callback,with right values, if the interruption is not triggered by the power button (home button for example)
		/*	if(_scummvm.surfaceWidth != 0 ||  _scummvm.surfaceHeight != 0)
				_scummvm.DotChangeSurface(_scummvm.surfaceHolder, _scummvm.surfaceFormat, _scummvm.surfaceWidth, _scummvm.surfaceHeight);*/
		}
	}

	@Override
	public void onPause() {
	//	Log.d(ScummVM.LOG_TAG, "onPause");

		super.onPause();

		if (_scummvm != null)
		{
			_scummvm.setPause(true);
			
			// DotEmu hack to call surfaceDestroyed when going to sleep via power button
			//_scummvm.DotDestroySurface();
		}
	}

	@Override
	public void onStop() {
	//	Log.d(ScummVM.LOG_TAG, "onStop");

		super.onStop();
	}

	@Override
	public void onDestroy() {
	//	Log.d(ScummVM.LOG_TAG, "onDestroy");

		super.onDestroy();

		if (_events != null) {
			_events.sendQuitEvent();

			try {
				// 1s timeout
				_scummvm_thread.join(1000);
			} catch (InterruptedException e) {
				Log.i(ScummVM.LOG_TAG, "Error while joining ScummVM thread", e);
			}

			_scummvm = null;
		}
	}

	@Override
	public boolean onTrackballEvent(MotionEvent e) {
		if (_events != null)
			return _events.onTrackballEvent(e);

		return false;
	}

	private void showKeyboard(boolean show) {
		
		SurfaceView main_surface = (SurfaceView)findViewById(R.id.main_surface);
		InputMethodManager imm = (InputMethodManager)
			getSystemService(INPUT_METHOD_SERVICE);
	//	Log.i("dotemu","show keyboad java");
		if (show)
			imm.showSoftInput(main_surface, InputMethodManager.SHOW_IMPLICIT);
		else
			imm.hideSoftInputFromWindow(main_surface.getWindowToken(),
										InputMethodManager.HIDE_IMPLICIT_ONLY);
	}
}
